#include <iostream>
#include <geos.h>
#include <BufferParameters.h>
#include <BufferBuilder.h>

void mitreBug()
{
   //build geometry
   std::vector<Coordinate> coordsVec;
   coordsVec.emplace_back(1, 0);
   coordsVec.emplace_back(0, 0);
   coordsVec.emplace_back(0, 1);
   GeometryFactory::unique_ptr gFactory = GeometryFactory::create();
   const CoordinateSequenceFactory* csFactory = gFactory->getCoordinateSequenceFactory();
   CoordinateSequence* tempSequence = csFactory->create(&coordsVec);
   LineString* elbow = gFactory->createLineString(*tempSequence);
   tempSequence = nullptr; // memory leak here is unrelated to current bug.  See ticket 325 for more info.
   std::cout << elbow->toString();

   //compute mitred corner
   using namespace geos::operation::buffer;
   double approxMitreLimitForBevel = 1 / sqrt(2);
   BufferParameters mitreParams(0, BufferParameters::CAP_FLAT, BufferParameters::JOIN_MITRE, approxMitreLimitForBevel);
   BufferBuilder mitreBufferBuilder(mitreParams);
   Geometry* mitred = mitreBufferBuilder.bufferLineSingleSided(elbow, 2, true);

   //compute beveled corner
   BufferParameters bevelParams(0, BufferParameters::CAP_FLAT, BufferParameters::JOIN_BEVEL, 1);
   BufferBuilder bevelBufferBuilder(bevelParams);
   Geometry* beveled = bevelBufferBuilder.bufferLineSingleSided(elbow, 2, true);


   //show difference
   //Expected these two to be the same.  Why are they different?
   std::cout << beveled->toString() << std::endl; // bevel's bevel: looks good! :)
   std::cout << mitred->toString() << std::endl; // mitre's bevel: edges are not parallel to input :(

   //clean up geometries
   gFactory->destroyGeometry(beveled); beveled = nullptr;
   gFactory->destroyGeometry(mitred); mitred = nullptr;
   gFactory->destroyGeometry(elbow); elbow = nullptr;

}

int main()
{
   mitreBug();

   return 0;
}
